#!/usr/bin/python3
# -*- coding: utf-8 -*-
import tkinter as Tk
import numpy as np
import time
from PIL import Image, ImageTk


class GestureGUI(object):

    def __init__(self,  sensor):

        self.sensor = sensor
        header, data = sensor.get_ranges()

        # Sensor Parameters
        self.bidirectional_range = 170
        self.presence_range = 250
        self.presence_time = 0.3
        self.engagement_trigger_range = 80
        self.engagement_trigger_time = 2
        self.disengagement_trigger_time = 3
        self.swipe_max = 45
        self.swipe_min = 20
        self.validation_range = 15
        self.validation_engage = 0.3

        # Value for minimal distance between threshold (cm)
        self.between_swipe_distance = 10
        self.between_validation_swipe_distance = 5
        self.between_swipe_engagement = 25
        self.swipe_min_distance_threshold = 20

        # Conversion factor
        self.conversion_factor_s_to_ms = 1000
        self.conversion_factor_cm_to_mm = 10
        self.conversion_factor_m_to_cm = 100
        self.conversion_factor_mm_to_cm = 1/10
        self.conversion_factor_ms_to_s = 1/1000

        # Retrieve internal sensor parameters
        self.get_parameter()

        self.presence_check = False
        # GUI init
        self.top = Tk.Tk()
        self.top.title('Terabee Evo Swipe Plus')

        # Create Labels for bidirectional
        self.bidirectional_label = Tk.Label(self.top, text='Bidirectional People Traffic Detection', font=("Helvetica", 17))
        self.bidirectional_label.grid(row=1, column=0, padx=10, pady=1)
        self.bidirectional_label_results = Tk.Label(self.top, text='', font=("Helvetica 16 bold"))
        self.bidirectional_label_results.grid(row=2, column=0, padx=10, pady=3)
        # Create slider for bidirectional counting
        self.bidirectional_slider = Tk.LabelFrame(self.top, text='Set Bidirectional People Traffic range detection (cm)', font=("Helvetica", 13))
        self.bidirectional_slider.grid(row=3, column=0, padx=10, pady=2)
        self.bidirectional_range_slider = Tk.Scale(self.bidirectional_slider, from_=60, to=170, orient=Tk.HORIZONTAL, length=300,
                                        resolution=10, command=self.update_bidirectional_max_distance)
        self.bidirectional_range_slider.set(self.bidirectional_range)
        self.bidirectional_range_slider.grid(row=3, column=1, padx=10, pady=2)

        # Presence check box to activate or deactivate presence mode
        self.presence_detector_on = Tk.BooleanVar()
        if header == b"DD":
            self.presence_detector_on.set(False)

        else:
            self.presence_detector_on.set(True)
        self.presence_validator = Tk.Checkbutton(self.top, text='Activate Presence Detection / Deactivate Bidirectional',
                                                 font=("Helvetica", 13), var=self.presence_detector_on)
        self.presence_validator.grid(row=4, column=0, padx=10, pady=2)

        # Create Labels for Presence
        self.presence_label = Tk.Label(self.top, text='Presence Detection', font=("Helvetica", 17))
        self.presence_label.grid(row=5, column=0, padx=10, pady=1)
        self.presence_counter_results = Tk.Label(self.top, text='', font=("Helvetica 16 bold"))
        self.presence_counter_results.grid(row=6, columnspan=1, padx=10, pady=2)
        self.presence_label_results = Tk.Label(self.top, text='', font=("Helvetica 16 bold"))
        self.presence_label_results.grid(row=7, columnspan=1, padx=10, pady=2)
        # Create slider for presence range threshold
        self.presence_slider = Tk.LabelFrame(self.top, text='Set Presence range detection (cm)',
                                       font=("Helvetica", 13))
        self.presence_slider.grid(row=8, column=0, padx=10, pady=1)
        self.presence_range_slider = Tk.Scale(self.presence_slider, from_=20, to=300, orient=Tk.HORIZONTAL, length=300,
                                        resolution=10)
        self.presence_range_slider.bind("<ButtonRelease-1>", self.update_presence_parameters)
        self.presence_range_slider.set(self.presence_range)
        self.presence_range_slider.grid(row=8, column=1, pady=1)
        # Create slider for presence time
        self.presence_time_threshold_slider = Tk.LabelFrame(self.top, text='Set time threshold to detect Presence (s)',
                                             font=("Helvetica", 13))
        self.presence_time_threshold_slider.grid(row=9, column=0, padx=10, pady=1)
        self.presence_time_slider = Tk.Scale(self.presence_time_threshold_slider, from_=0.1, to=3, orient=Tk.HORIZONTAL,
                                              length=300,
                                              resolution=0.1)

        self.presence_time_slider.bind("<ButtonRelease-1>", self.update_presence_parameters)
        self.presence_time_slider.set(self.presence_time)
        self.presence_time_slider.grid(row=9, column=1, pady=2)

        # Create Labels for engagement/disengagement
        self.engagement_status = Tk.Label(self.top, text='Engagement Trigger Status', font=("Helvetica", 17))
        self.engagement_status.grid(row=11, column=0, padx=10, pady=2)
        self.engagement_status_results = Tk.Label(self.top, text='', font=("Helvetica 16 bold"))
        self.engagement_status_results.grid(row=12, column=0, columnspan=1, padx=10, pady=2)
        # Create Slider for engagement/disengagement
        self.engagement_distance_slider_label = Tk.LabelFrame(self.top, text='Set Engagement range trigger (cm)', font=("Helvetica", 13))
        self.engagement_distance_slider_label.grid(row=13, column=0, padx=10, pady=2)
        self.engagement_distance_slider = Tk.Scale(self.engagement_distance_slider_label, from_=50, to=150,
                                                   orient=Tk.HORIZONTAL, command=self.update_engagement_parameters,
                                                   length=300, resolution=10)
        self.engagement_distance_slider.set(self.engagement_trigger_range)
        self.engagement_distance_slider.grid(row=13, column=1, padx=10, pady=1)

        self.engagement_time_slider_label = Tk.LabelFrame(self.top, text='Set Engagement time trigger (s)', font=("Helvetica", 13))
        self.engagement_time_slider_label.grid(row=14, column=0, padx=10)
        self.engagement_time_slider = Tk.Scale(self.engagement_time_slider_label, from_=1, to=10,
                                               orient=Tk.HORIZONTAL, command=self.update_engagement_parameters,
                                               length=200)
        self.engagement_time_slider.set(self.engagement_trigger_time)
        self.engagement_time_slider.grid(row=14, column=1, padx=10, pady=1)

        self.disengagement_time_slider_label = Tk.LabelFrame(self.top, text='Set Disengagement time trigger (s)', font=("Helvetica", 13))
        self.disengagement_time_slider_label.grid(row=15, column=0, padx=10, pady=2)
        self.disengagement_time_slider = Tk.Scale(self.disengagement_time_slider_label, from_=1, to=10,
                                                  orient=Tk.HORIZONTAL, command=self.update_engagement_parameters,
                                                  length=200)
        self.disengagement_time_slider.set(self.disengagement_trigger_time)
        self.disengagement_time_slider.grid(row=15, column=1, padx=10, pady=1)

        # Create Labels for Swipe
        self.swipe_label = Tk.Label(self.top, text='Last Gesture', font=("Helvetica", 17))
        self.swipe_label.grid(row=1, column=1, columnspan=1, padx=10, pady=2)
        self.swipe_label_results = Tk.Label(self.top, text='-', font=("Helvetica 16 bold"))
        self.swipe_label_results.grid(row=2, column=1, columnspan=1, padx=10, pady=3)
        # Create Slider for Swipe
        self.swipe_min_distance_slider_label = Tk.LabelFrame(self.top, text='Set Swipe minimum distance detection (cm)', font=("Helvetica", 13))
        self.swipe_min_distance_slider_label.grid(row=3, column=1, padx=10, pady=1)
        self.swipe_min_distance_slider = Tk.Scale(self.swipe_min_distance_slider_label, from_=10, to=40,
                                                  orient=Tk.HORIZONTAL,
                                                  length=300, resolution=5, command=self.update_swipe_min_distance)
        self.swipe_min_distance_slider.set(self.swipe_min)
        self.swipe_min_distance_slider.grid(row=4, column=1, padx=10, pady=1)
        self.swipe_max_distance_slider_label = Tk.LabelFrame(self.top, text='Set Swipe maximum distance detection (cm)', font=("Helvetica", 13))
        self.swipe_max_distance_slider_label.grid(row=5, column=1, padx=10, pady=2)
        self.swipe_max_distance_slider = Tk.Scale(self.swipe_max_distance_slider_label, from_=20, to=50,
                                                  orient=Tk.HORIZONTAL,
                                                  length=300, resolution=5, command=self.update_swipe_max_distance)
        self.swipe_max_distance_slider.set(self.swipe_max)
        self.swipe_max_distance_slider.grid(row=6, column=1, padx=10, pady=1)

        # Create Labels for Validation
        self.validation_label = Tk.Label(self.top, text='Validation Status', font=("Helvetica", 17))
        self.validation_label.grid(row=7, column=1, columnspan=1, padx=10, pady=2)
        self.validation_label_status = Tk.Label(self.top, text='-', font=("Helvetica 16 bold"))
        self.validation_label_status.grid(row=8, column=1, columnspan=1, padx=10)
        # Create Sliders for Validation
        self.validation_distance_label = Tk.LabelFrame(self.top, text='Set Validation distance detection (cm)', font=("Helvetica", 13))
        self.validation_distance_label.grid(row=9, column=1, padx=10, pady=2)
        self.validation_distance = Tk.Scale(self.validation_distance_label, from_=1, to=15, orient=Tk.HORIZONTAL,
                                                  length=300, resolution=1, command=self.update_validation_parameters)
        self.validation_distance.set(self.validation_range)
        self.validation_distance.grid(row=10, column=1, padx=10, pady=1)
        self.validation_time_label = Tk.LabelFrame(self.top, text='Set Validation time detection (s)', font=("Helvetica", 13))
        self.validation_time_label.grid(row=11, column=1, padx=10, pady=1)
        self.validation_time = Tk.Scale(self.validation_time_label, from_=0.1, to=3, orient=Tk.HORIZONTAL,
                                            length=300, resolution=0.1, command=self.update_validation_parameters)
        self.validation_time.set(self.validation_engage)
        self.validation_time.grid(row=12, column=1, padx=10, pady=1)

        self.button_label = Tk.Label(self.top, text='Button to reset Counters', font=("Helvetica", 17))
        self.button_label.grid(row=13, columnspan=1, column=1, padx=10, pady=1)
        self.reset_bidirectional_counting_button = Tk.Button(self.top, text='Reset Bidirectional Counter', font=("Helvetica", 13), command=self.reset_bidirectional)
        self.reset_bidirectional_counting_button.grid(row=14, column=1, padx=10, pady=1)
        self.reset_detection_button = Tk.Button(self.top, text='Reset Presence Counter', font=("Helvetica", 13), command=self.reset_detection)
        self.reset_detection_button.grid(row=15, column=1, padx=10, pady=1)

    def update_bidirectional_max_distance(self, *args):
        bidirectional_max_limit = int(self.bidirectional_range_slider.get() * self.conversion_factor_cm_to_mm)
        bidirectional_min_limit = 30 * self.conversion_factor_cm_to_mm
        self.sensor.set_bidirectional_range(bidirectional_max_limit, bidirectional_min_limit)

    def update_presence_parameters(self, *args):
        presence_threshold_limit = int(self.presence_range_slider.get() * self.conversion_factor_cm_to_mm)
        presence_time_detection = int(self.presence_time_slider.get() * self.conversion_factor_s_to_ms)
        if presence_time_detection > int(self.engagement_time_slider.get() * self.conversion_factor_s_to_ms):
            self.engagement_time_slider.set(presence_time_detection/1000+1)
        self.sensor.set_presence_params(presence_threshold_limit, presence_time_detection)

    def update_engagement_parameters(self, *args):
        engagement_threshold = int(self.engagement_distance_slider.get())
        engagement_time = int(self.engagement_time_slider.get())
        disengagement_time = int(self.disengagement_time_slider.get())
        if engagement_threshold < (self.swipe_max_distance_slider.get() + self.between_swipe_engagement):
            self.swipe_max_distance_slider.set(engagement_threshold - self.between_swipe_engagement)
        if engagement_time < self.presence_time_slider.get():
            self.presence_time_slider.set(int(engagement_time))
        engagement_threshold = engagement_threshold * self.conversion_factor_cm_to_mm
        self.sensor.set_engagement_params(engagement_threshold, engagement_time, disengagement_time)

    def update_swipe_min_distance(self, *args):
        swipe_min_distance = int(self.swipe_min_distance_slider.get())
        swipe_max_distance = int(self.swipe_max_distance_slider.get())
        if swipe_min_distance < (self.validation_distance.get() + self.between_validation_swipe_distance):
            self.validation_distance.set(swipe_min_distance - self.between_validation_swipe_distance)
        if swipe_min_distance > (swipe_max_distance - self.between_swipe_distance):
            swipe_max_distance = swipe_min_distance + self.between_swipe_distance
            self.swipe_max_distance_slider.set(swipe_min_distance + self.between_swipe_distance)
        swipe_min_distance = swipe_min_distance * self.conversion_factor_cm_to_mm
        swipe_max_distance = swipe_max_distance * self.conversion_factor_cm_to_mm
        self.sensor.set_swipe_min_and_max(swipe_max_distance, swipe_min_distance)
        self.swipe_min_distance_threshold = swipe_min_distance

    def update_swipe_max_distance(self, *args):
        swipe_min_distance = int(self.swipe_min_distance_slider.get())
        swipe_max_distance = int(self.swipe_max_distance_slider.get())
        if swipe_max_distance > (self.engagement_distance_slider.get() - self.between_swipe_engagement):
            self.engagement_distance_slider.set(swipe_max_distance + self.between_swipe_engagement)
        if swipe_max_distance < (swipe_min_distance + self.between_swipe_distance):
            swipe_min_distance = swipe_max_distance - self.between_swipe_distance
            self.swipe_min_distance_slider.set(swipe_max_distance - self.between_swipe_distance)
        swipe_min_distance = swipe_min_distance * self.conversion_factor_cm_to_mm
        swipe_max_distance = swipe_max_distance * self.conversion_factor_cm_to_mm
        self.sensor.set_swipe_min_and_max(swipe_max_distance, swipe_min_distance)

    def update_validation_parameters(self, *args):
        validation_range_threshold_limit = int(self.validation_distance.get())
        validation_time_threshold = int(self.validation_time.get() * self.conversion_factor_s_to_ms)
        if validation_range_threshold_limit > (self.swipe_min_distance_slider.get() - self.between_validation_swipe_distance):
            self.swipe_min_distance_slider.set(validation_range_threshold_limit + self.between_validation_swipe_distance)
        validation_range_threshold_limit = validation_range_threshold_limit * self.conversion_factor_cm_to_mm
        self.sensor.set_validation_params(validation_range_threshold_limit, validation_time_threshold)

    def reset_bidirectional(self, *args):
        self.sensor.reset_bidirectional_counter()

    def reset_detection(self, *args):
        self.sensor.reset_detection_counter()

    def get_parameter(self):

        dictionary_parameters = self.sensor.get_sensor_parameters()
        if dictionary_parameters is not None:
            self.bidirectional_range = dictionary_parameters["bidirectional_range"][0] * self.conversion_factor_mm_to_cm
            self.presence_range = dictionary_parameters["presence_range"][0] * self.conversion_factor_mm_to_cm
            self.presence_time = dictionary_parameters["presence_time"][0] * self.conversion_factor_ms_to_s
            self.engagement_trigger_range = dictionary_parameters["engagement_range"][0] * self.conversion_factor_mm_to_cm
            self.engagement_trigger_time = dictionary_parameters["engagement_time"]
            self.disengagement_trigger_time = dictionary_parameters["disengagement_time"]
            self.swipe_max = dictionary_parameters["swipe_max"][0] * self.conversion_factor_mm_to_cm
            self.swipe_min = dictionary_parameters["swipe_min"][0] * self.conversion_factor_mm_to_cm
            self.validation_range = dictionary_parameters["validation_range"][0] * self.conversion_factor_mm_to_cm
            self.validation_engage = dictionary_parameters["validation_time"][0] * self.conversion_factor_ms_to_s

    def update_gui(self, bidirectional_count, last_gesture, presence_status, presence_count, engagement_status, validation_status):

        if self.presence_detector_on.get():
            if self.presence_check is False:
                self.sensor.set_presence_mode()
            self.presence_check = True
            bidirectional_count = "Not Activated"
            if presence_status:
                self.presence_label_results.config(text=presence_status)
        else:
            presence_count = "Not activated"
            self.bidirectional_label_results.config(text=bidirectional_count)
            if self.presence_check:
                self.sensor.set_bidirectional_mode()
            self.presence_check = False

        self.bidirectional_label_results.config(text=bidirectional_count)
        self.presence_counter_results.config(text=presence_count)
        self.engagement_status_results.config(text=engagement_status)
        self.swipe_label_results.config(text=last_gesture)
        self.validation_label_status.config(text=validation_status)
        self.presence_label_results.config(text=presence_status)

        self.top.update()
